@@ -32,11 +32,27 @@ def checkFileExist(file):
     if not os.path.exists(os.path.abspath(file)):
         sys.exit('%s not found' % file)
 
+def checkRefspec(coreDir, coreSet, refspecList):
+    taxaFile = '%s/core_orthologs/%s/done.txt' % (coreDir, coreSet)
+    checkFileExist(taxaFile)
+    allTaxa = fcatO.readFile(taxaFile)
+    invalid = []
+    for refspec in refspecList:
+        if not refspec in allTaxa:
+            invalid.append(refspec)
+    if len(invalid) > 0:
+        print('ERROR: Invalid refspec found: %s' % '; '.join(invalid))
+        print('Please use fcat.showTaxa for a list of valid reference taxa!')
+        sys.exit()
+
 def fcat(args):
     # calculate group specific cutoffs
     # print('##### Calculating group specific cutoffs...')
     # fcatC.calcGroupCutoff(args)
 
+    # check for valid refspec
+    checkRefspec(args.coreDir, args.coreSet, str(args.refspecList).split(","))
+
     # search for orthologs and create phylognetic profile files
     print('##### Searching for orthologs...')
     fcatO.searchOrtho(args)
@@ -65,11 +81,12 @@ def fcat(args):
     print('##### Generating reports...')
     fcatR.assessCompteness(args)
 
-    # merge phyloprofile output
+    # merge phyloprofile
+    print('##### Merging phylogenetic profiles of all query taxa...')
     fcatM.mergePP(args)
 
 def main():
-    version = '0.0.9'
+    version = '0.0.12'
     parser = argparse.ArgumentParser(description='You are running fcat version ' + str(version) + '.')
     required = parser.add_argument_group('required arguments')
     optional = parser.add_argument_group('optional arguments')
@@ -87,6 +104,7 @@ def main():
     optional.add_argument('--cpus', help='Number of CPUs used for annotation. Default = 4', action='store', default=4, type=int)
     optional.add_argument('--force', help='Force overwrite existing ortholog search and assessment output', action='store_true', default=False)
     optional.add_argument('--forceProfile', help='Force overwrite phylogenetic profiles output', action='store_true', default=False)
+    optional.add_argument('--noDomain', help='Not output feature domain files', action='store_true', default=False)
     optional.add_argument('--keep', help='Keep temporary phyloprofile data', action='store_true', default=False)
     optional.add_argument('--bidirectional', help=argparse.SUPPRESS, action='store_true', default=False)
 
