@@ -35,7 +35,7 @@ def checkFileExist(file, msg):
 def readFile(file):
     if os.path.exists(file):
         with open(file, 'r') as f:
-            lines = f.readlines()
+            lines = f.read().splitlines()
             f.close()
             return(lines)
     else:
@@ -52,29 +52,47 @@ def mergePP(args):
     for coreSpec in os.listdir('%s/blast_dir' % coreDir):
         coreTaxaId.append(coreSpec.split('@')[1])
     mode1out = []
-    mode23out = []
+    mode2out = []
+    mode3out = []
+    domainOut = []
     for query in os.listdir(outDir):
         if os.path.isdir(outDir + '/' + query):
             if not query.split('@')[1] in coreTaxaId:
                 if os.path.exists('%s/%s/phyloprofileOutput/mode1.phyloprofile' % (outDir, query)):
                     for line in readFile('%s/%s/phyloprofileOutput/mode1.phyloprofile' % (outDir, query)):
-                        if not line.strip() in mode1out:
-                            mode1out.append(line.strip())
-                if os.path.exists('%s/%s/phyloprofileOutput/mode23.phyloprofile' % (outDir, query)):
-                    for line in readFile('%s/%s/phyloprofileOutput/mode23.phyloprofile' % (outDir, query)):
-                        if not line.strip() in mode23out:
-                            mode23out.append(line.strip())
+                        if not line in mode1out:
+                            mode1out.append(line)
+                if os.path.exists('%s/%s/phyloprofileOutput/mode2.phyloprofile' % (outDir, query)):
+                    for line in readFile('%s/%s/phyloprofileOutput/mode2.phyloprofile' % (outDir, query)):
+                        if not line in mode2out:
+                            mode2out.append(line)
+                if os.path.exists('%s/%s/phyloprofileOutput/mode3.phyloprofile' % (outDir, query)):
+                    for line in readFile('%s/%s/phyloprofileOutput/mode3.phyloprofile' % (outDir, query)):
+                        if not line in mode3out:
+                            mode3out.append(line)
+                if os.path.exists('%s/%s/phyloprofileOutput/mode123.domains' % (outDir, query)):
+                    for line in readFile('%s/%s/phyloprofileOutput/mode123.domains' % (outDir, query)):
+                        if not line in domainOut:
+                            domainOut.append(line)
     if len(mode1out) > 0:
         mode1File = open('%s/mode1.phyloprofile' % (outDir), 'w')
-        mode1File.write('\n'.join(mode1out))
+        mode1File.write('%s\n' % '\n'.join(mode1out))
         mode1File.close()
-    if len(mode23out) > 0:
-        mode23File = open('%s/mode23.phyloprofile' % (outDir), 'w')
-        mode23File.write('\n'.join(mode23out))
-        mode23File.close()
+    if len(mode2out) > 0:
+        mode2File = open('%s/mode2.phyloprofile' % (outDir), 'w')
+        mode2File.write('%s\n' % '\n'.join(mode2out))
+        mode2File.close()
+    if len(mode3out) > 0:
+        mode3File = open('%s/mode3.phyloprofile' % (outDir), 'w')
+        mode3File.write('%s\n' % '\n'.join(mode3out))
+        mode3File.close()
+    if len(domainOut) > 0:
+        domainFile = open('%s/mode123.domains' % (outDir), 'w')
+        domainFile.write('%s\n' % '\n'.join(domainOut))
+        domainFile.close()
 
 def main():
-    version = '0.0.9'
+    version = '0.0.12'
     parser = argparse.ArgumentParser(description='You are running fcat version ' + str(version) + '.')
     parser.add_argument('-d', '--coreDir', help='Path to core set directory, where folder core_orthologs can be found', action='store', default='', required=True)
     parser.add_argument('-c', '--coreSet', help='Name of core set, which is subfolder within coreDir/core_orthologs/ directory', action='store', default='', required=True)
