@@ -26,8 +26,6 @@ import shutil
 from tqdm import tqdm
 import time
 import datetime
-# import tarfile
-# import re
 
 def checkFileExist(file, msg):
     if not os.path.exists(os.path.abspath(file)):
@@ -39,7 +37,7 @@ def roundTo4(number):
 def readFile(file):
     if os.path.exists(file):
         with open(file, 'r') as f:
-            lines = f.readlines()
+            lines = f.read().splitlines()
             f.close()
             return(lines)
     else:
@@ -145,7 +143,7 @@ def checkRefspec(refspecList, groupFa):
 def readRefspecFile(refspecFile):
     groupRefspec = {}
     for line in readFile(refspecFile):
-        groupRefspec[line.split('\t')[0]] = line.split('\t')[1].strip()
+        groupRefspec[line.split('\t')[0]] = line.split('\t')[1]
     return(groupRefspec)
 
 def prepareJob(coreDir, coreSet, queryID, refspecList, outDir, blastDir, annoDir, annoQuery, force, cpus):
@@ -455,7 +453,7 @@ def searchOrtho(args):
         pool.close()
         pool.join()
 
-    if not status == 2:
+    # if not status == 2:
         print('Calculating pairwise FAS scores between query orthologs and sequences of refspec...')
         missing = calcFAS(coreDir, outDir, coreSet, queryID, annoDir, cpus, force)
         print('Calculating FAS scores between query orthologs and all sequences in each core group...')
@@ -487,7 +485,7 @@ def searchOrtho(args):
     print('Done! Check output in %s' % fcatOut)
 
 def main():
-    version = '0.0.9'
+    version = '0.0.12'
     parser = argparse.ArgumentParser(description='You are running fcat version ' + str(version) + '.')
     required = parser.add_argument_group('required arguments')
     optional = parser.add_argument_group('optional arguments')
